import { CronJob } from "cron";
import useClient from "../utils/use-client.js";
import UserModel from "../models/UserModel.js";
import { createEmbed } from "../utils/create-embed.js";
import logger from "./logger.js";
import { getNoun } from "./functions.js";

const checkBirthdays = async () => {
	const client = useClient();

	const guilds = client.guilds.cache.values();
	const users = await client.adapter.find(UserModel, { birthdate: { $gt: 1 } });

	const currentData = new Date();
	const currentYear = currentData.getFullYear();
	const currentMonth = currentData.getMonth();
	const currentDate = currentData.getDate();

	for (const guild of guilds) {
		try {
			const data = await client.getGuildData(guild.id);
			const channel = data.plugins.birthdays ? await client.channels.fetch(data.plugins.birthdays) : null;

			if (!channel) return;

			const userIDs = users.filter(u => guild.members.cache.has(u.id)).map(u => u.id);

			await Promise.all(
				userIDs.map(async userID => {
					const user = users.find(u => u.id === userID);
					const userData = new Date(user.birthdate).getFullYear() <= 1970 ? new Date(user.birthdate * 1000) : new Date(user.birthdate);
					const userYear = userData.getFullYear();
					const userMonth = userData.getMonth();
					const userDate = userData.getDate();

					const age = currentYear - userYear;

					if (userDate === currentDate && userMonth === currentMonth) {
						const embed = createEmbed({
							author: client.user.username,
							fields: [
								{
									name: client.translate("economy/birthdate:HAPPY_BIRTHDAY", {
										lng: data.language,
									}),
									value: client.translate("economy/birthdate:HAPPY_BIRTHDAY_MESSAGE", {
										lng: data.language,
										user: user.id,
										age: `**${age}** ${getNoun(age, [
											client.translate("misc:NOUNS:AGE:1", null, data.language),
											client.translate("misc:NOUNS:AGE:2", null, data.language),
											client.translate("misc:NOUNS:AGE:5", null, data.language),
										])}`,
									}),
								},
							],
						});

						await channel.send({ embeds: [embed] }).then(m => m.react(" "));
					}
				}),
			);
		} catch (error) {
			logger.error(error);
		}
	}
};

export async function init() {
	new CronJob("0 5 * * *", checkBirthdays(), null, true, "Europe/Moscow");
}
